# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from fabric_cm.credmgr.swagger_server.models.base_model_ import Model
from fabric_cm.credmgr.swagger_server import util


class Request(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    def __init__(self, refresh_token: str=None):  # noqa: E501
        """Request - a model defined in Swagger

        :param refresh_token: The refresh_token of this Request.  # noqa: E501
        :type refresh_token: str
        """
        self.swagger_types = {
            'refresh_token': str
        }

        self.attribute_map = {
            'refresh_token': 'refresh_token'
        }
        self._refresh_token = refresh_token

    @classmethod
    def from_dict(cls, dikt) -> 'Request':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The request of this Request.  # noqa: E501
        :rtype: Request
        """
        return util.deserialize_model(dikt, cls)

    @property
    def refresh_token(self) -> str:
        """Gets the refresh_token of this Request.

        Refresh Token  # noqa: E501

        :return: The refresh_token of this Request.
        :rtype: str
        """
        return self._refresh_token

    @refresh_token.setter
    def refresh_token(self, refresh_token: str):
        """Sets the refresh_token of this Request.

        Refresh Token  # noqa: E501

        :param refresh_token: The refresh_token of this Request.
        :type refresh_token: str
        """
        if refresh_token is None:
            raise ValueError("Invalid value for `refresh_token`, must not be `None`")  # noqa: E501

        self._refresh_token = refresh_token
